/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc_pkcs_5_2.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tomcrypt.h"
#include "tc_priv.h"

/*
 * Key deriving function (PBKDF2 as specified in PKCS #5 v2)
 */

BOOL __stdcall TC_DeriveKey(void *pKey, size_t cbKeySize,
  const void *pPassword, size_t cbPasswordSize,
  const void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  PPKCS5_2_BUFFER pBuffer)
{
    const struct ltc_hash_descriptor *pdHash;
    unsigned long  ulKeySize;

    switch (uHash) {
#ifdef RIPEMD160
    case TC_RIPEMD160:
        if (uIterationCount == 0) {
            uIterationCount = TC_RIPEMD160_ITERATION_COUNT;
        }
        pdHash = &rmd160_desc;
        break;
#endif
#ifdef SHA1
    case TC_SHA1:
        if (uIterationCount == 0) {
            uIterationCount = TC_SHA1_ITERATION_COUNT;
        }
        pdHash = &sha1_desc;
        break;
#endif
#ifdef WHIRLPOOL
    case TC_WHIRLPOOL:
        if (uIterationCount == 0) {
            uIterationCount = TC_WHIRLPOOL_ITERATION_COUNT;
        }
        pdHash = &whirlpool_desc;
        break;
#endif
    default:
        return FALSE;
    }

    ulKeySize = (unsigned long) cbKeySize;
    if (tc_pkcs_5_alg2(pPassword, (unsigned long)cbPasswordSize, pSalt,
      (unsigned long)cbSaltSize, uIterationCount, pdHash, pKey, &ulKeySize, pBuffer) !=
      CRYPT_OK) {
        return FALSE;
    }

    return ulKeySize == cbKeySize;
}

BOOL __stdcall TC_DeriveNonWeakKey(PTC_KEY pKey,
  const void *pPassword, size_t cbPasswordSize,
  void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  unsigned uCipher, PPKCS5_2_BUFFER pBuffer)
{
    unsigned __int8 *pbSalt = (unsigned __int8 *) pSalt;
    unsigned i;

    for (i = 256; --i; ) {
        if (!TC_DeriveKey(pKey, sizeof(TC_KEY), pPassword,
          cbPasswordSize, pSalt, cbSaltSize,
          uHash, uIterationCount, pBuffer)) {
            return FALSE;
        }
        if (!TC_KeyIsWeak(pKey, uCipher)) {
            return TRUE;
        }

        if (cbSaltSize == 0) {
            return FALSE;
        }

        pbSalt[0]++;
    }

    return FALSE;
}
